import 'package:moor/moor.dart';
import 'package:unify_api/unify.dart' as api;

import '../database.dart';
import '../tables/contacts.dart';

part 'contact.g.dart';

@UseDao(tables: [
  ContactRecords
], queries: {
  'contactsById':
      'SELECT * FROM contacts WHERE user_id = :userId AND contact_id = :contactId LIMIT 1',
  'contactsByUser':
      'SELECT * FROM contacts WHERE user_id = :userId ORDER BY last_name ASC',
})
class ContactsDao extends DatabaseAccessor<DatabaseService>
    with _$ContactsDaoMixin {
  ContactsDao(DatabaseService db) : super(db);

  Future batchUpdate(String userId, List<api.Contact> contacts) async {
    final _items = <ContactRecordsCompanion>[];
    final _updated = DateTime.now();
    for (var contact in contacts) {
      final _item = ContactRecordsCompanion(
          contactId: Value(contact.id),
          userId: Value(userId),
          firstName: addField<String>(contact?.firstName),
          lastName: addField<String>(contact?.lastName),
          cellPhone: addField<String>(contact?.cellPhone.toString()),
          homePhone: addField<String>(contact?.homePhone.toString()),
          officePhone: addField<String>(contact?.officePhone.toString()),
          email: addField<String>(contact?.email),
          lastActivity: addField<String>(contact?.lastActivity),
          dateCreated: addField<DateTime>(contact?.dateCreated),
          lastUpdated: addField<DateTime>(_updated));
      _items.add(_item);
    }
    await into(contactRecords).insertAll(_items, orReplace: true);
    await cleanContacts(userId, _updated);
  }

  Future<List<Contact>> searchContacts(String userId, String query) {
    return (select(contactRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) {
            if (num.tryParse(query) != null) {
              final _num = query
                  .replaceAll('-', '')
                  .replaceAll(' ', '')
                  .replaceAll('(', '')
                  .replaceAll(')', '')
                  .replaceAll(';', '');
              return or(
                  or(
                    l.cellPhone.like('%$_num%'),
                    l.homePhone.like('%$_num%'),
                  ),
                  l.officePhone.like('%$_num%'));
            }
            return or(
                l.email.like('%$query%'),
                or(
                  l.firstName.like('%$query%'),
                  l.lastName.like('%$query%'),
                ));
          })
          ..orderBy([
            (u) => OrderingTerm(expression: u.lastName, mode: OrderingMode.asc),
            (u) =>
                OrderingTerm(expression: u.firstName, mode: OrderingMode.asc),
          ]))
        .get();
  }

  Future<Contact> getLastContact(String userID) async {
    final _items = await (select(contactRecords)
          ..where((l) => l.userId.equals(userID))
          ..orderBy([
            (u) => OrderingTerm(
                expression: u.lastUpdated, mode: OrderingMode.desc),
          ]))
        .get();
    if (_items != null && _items.isNotEmpty) {
      return _items.first;
    }
    return null;
  }

  Future<Contact> getContact(String userId, String contactId) async {
    final _contacts = await contactsById(userId, contactId);
    if (_contacts != null && _contacts.isNotEmpty) {
      return _contacts.first;
    }
    return null;
  }

  Future<List<Contact>> getContacts(
      String userId, SortField sortField, bool sortDesc) async {
    final _items = await (select(contactRecords)
          ..where((l) => l.userId.equals(userId))
          ..orderBy([
            (u) {
              final _direction =
                  sortDesc ? OrderingMode.asc : OrderingMode.desc;
              switch (sortField) {
                case SortField.firstName:
                  return OrderingTerm(
                      expression: u.firstName, mode: _direction);
                case SortField.lastName:
                  return OrderingTerm(expression: u.lastName, mode: _direction);
                case SortField.email:
                  return OrderingTerm(expression: u.email, mode: _direction);
                case SortField.cellPhone:
                  return OrderingTerm(
                      expression: u.cellPhone, mode: _direction);
                case SortField.homePhone:
                  return OrderingTerm(
                      expression: u.homePhone, mode: _direction);
                case SortField.officePhone:
                  return OrderingTerm(
                      expression: u.officePhone, mode: _direction);
                case SortField.lastActivity:
                  return OrderingTerm(
                      expression: u.lastActivity, mode: _direction);
                case SortField.dateCreated:
                  return OrderingTerm(
                      expression: u.dateCreated, mode: _direction);
                default:
                  return OrderingTerm(
                      expression: u.lastUpdated, mode: _direction);
              }
            },
          ]))
        .get();
    if (_items != null && _items.isNotEmpty) {
      return _items;
    }
    return null;
  }

  Future resetContactsForUser(String userId) {
    return (delete(contactRecords)..where((c) => c.userId.equals(userId))).go();
  }

  Future cleanContacts(String userId, DateTime lastUpdated) async {
    return await (delete(contactRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) => l.lastUpdated.isBiggerThanValue(lastUpdated)))
        .go();
  }

  Future deleteContact(String userId, String contactId) async {
    return await (delete(contactRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) => l.contactId.equals(contactId)))
        .go();
  }
}
