import 'package:moor/moor.dart';
import 'package:unify_api/unify.dart' as api;

import '../database.dart';
import '../tables/leads.dart';

part 'lead.g.dart';

@UseDao(tables: [
  LeadRecords
], queries: {
  'leadsById':
      'SELECT * FROM leads WHERE user_id = :userId AND lead_id = :leadId LIMIT 1',
  'leadsByUser':
      'SELECT * FROM leads WHERE user_id = :userId ORDER BY last_name ASC'
})
class LeadsDao extends DatabaseAccessor<DatabaseService> with _$LeadsDaoMixin {
  LeadsDao(DatabaseService db) : super(db);

  Future batchUpdate(String userId, List<api.Lead> leads) async {
    final _items = <LeadRecordsCompanion>[];
    final _updated = DateTime.now();
    for (var lead in leads) {
      final _item = LeadRecordsCompanion(
          leadId: Value(lead.id),
          userId: Value(userId),
          contactId: addField<String>(lead?.contactId),
          firstName: addField<String>(lead?.firstName),
          lastName: addField<String>(lead?.lastName),
          cellPhone: addField<String>(lead?.cellPhone.toString()),
          homePhone: addField<String>(lead?.homePhone.toString()),
          officePhone: addField<String>(lead?.officePhone.toString()),
          email: addField<String>(lead?.email),
          lastActivity: addField<String>(lead?.lastActivity),
          dateCreated: addField<DateTime>(lead?.dateCreated),
          lastUpdated: Value(DateTime.now()));
      _items.add(_item);
    }
    await into(leadRecords).insertAll(_items, orReplace: true);
    await cleanLeads(userId, _updated);
  }

  Future<List<Lead>> searchLeads(String userId, String query) {
    return (select(leadRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) {
            if (num.tryParse(query) != null) {
              final _num = query
                  .replaceAll('-', '')
                  .replaceAll(' ', '')
                  .replaceAll('(', '')
                  .replaceAll(')', '')
                  .replaceAll(';', '');
              return or(
                  or(
                    l.cellPhone.like('%$_num%'),
                    l.homePhone.like('%$_num%'),
                  ),
                  l.officePhone.like('%$_num%'));
            }
            return or(
                l.email.like('%$query%'),
                or(
                  l.firstName.like('%$query%'),
                  l.lastName.like('%$query%'),
                ));
          })
          ..orderBy([
            (u) => OrderingTerm(expression: u.lastName, mode: OrderingMode.asc),
            (u) =>
                OrderingTerm(expression: u.firstName, mode: OrderingMode.asc),
          ]))
        .get();
  }

  Future<Lead> getLastLead(String userId) async {
    final _item = await (select(leadRecords)
          ..where((l) => l.userId.equals(userId))
          ..orderBy([
            (u) => OrderingTerm(
                expression: u.lastUpdated, mode: OrderingMode.desc),
          ]))
        .get();
    if (_item != null && _item.isNotEmpty) {
      return _item.first;
    }
    return null;
  }

  Future<Lead> getLead(String userId, String leadId) async {
    final _leads = await leadsById(userId, leadId);
    if (_leads != null && _leads.isNotEmpty) {
      return _leads.first;
    }
    return null;
  }

  Future resetLeadsForUser(String userId) {
    return (delete(leadRecords)..where((c) => c.userId.equals(userId))).go();
  }

  Future cleanLeads(String userId, DateTime lastUpdated) async {
    return await (delete(leadRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) => l.lastUpdated.isBiggerThanValue(lastUpdated)))
        .go();
  }

  Future deleteLead(String userId, String leadId) async {
    return (delete(leadRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) => l.leadId.equals(leadId)))
        .go();
  }
}
