import 'package:flutter/foundation.dart';
import 'package:moor/moor.dart';
import 'package:unify_api/unify.dart' as api;

import '../database.dart';
import '../tables/loan.dart';

part 'loan.g.dart';

@UseDao(
  tables: [LoanRecords],
  queries: {
    'loansById':
        'SELECT * FROM loans WHERE user_i_d = :userID AND loan_id = :loanID LIMIT 1',
    'loansByFolder':
        'SELECT * FROM loans WHERE user_i_d = :userID AND folder = :folder ORDER BY last_name ASC',
  },
)
class LoansDao extends DatabaseAccessor<DatabaseService> with _$LoansDaoMixin {
  LoansDao(DatabaseService db) : super(db);

  Future batchUpdate(
    String userID,
    List<api.Loan> loans, {
    api.LoansFolder folder = api.LoansFolder.all,
  }) async {
    final _items = <LoanRecordsCompanion>[];
    final _updated = DateTime.now();
    for (var loan in loans) {
      final _item = LoanRecordsCompanion(
        loanId: Value(loan.id),
        folder: addField<String>(describeEnum(folder),
            fallback: describeEnum(api.LoansFolder.all)),
        userID: Value(userID),
        firstName: addField<String>(loan?.firstName),
        lastName: addField<String>(loan?.lastName),
        cellPhone: addField<String>(phoneFormat(loan?.cellPhone)),
        homePhone: addField<String>(phoneFormat(loan?.homePhone)),
        officePhone: addField<String>(phoneFormat(loan?.homePhone)),
        email: addField<String>(loan?.email),
        dateClosed: addField<DateTime>(loan?.dateClosed),
        loanMilestone: addField<String>(loan?.loanMilestone),
        loanNumber: addField<String>(loan?.loanNumber),
        loanRate: addField<double>(loan?.loanRate),
        lastUpdated: addField<DateTime>(_updated),
      );
      _items.add(_item);
    }
    await into(loanRecords).insertAll(_items, orReplace: true);
    await cleanLoans(userID, _updated, folder: folder);
  }

  Future<List<Loan>> searchLoans(
    String userID,
    String query, {
    api.LoansFolder folder = api.LoansFolder.all,
  }) {
    return (select(loanRecords)
          ..where((l) => l.userID.equals(userID))
          ..where((l) => l.folder.equals(describeEnum(folder)))
          ..where((l) {
            if (num.tryParse(query) != null) {
              final _num = query
                  .replaceAll('-', '')
                  .replaceAll(' ', '')
                  .replaceAll('(', '')
                  .replaceAll(')', '')
                  .replaceAll(';', '');
              return or(
                  or(
                    l.loanNumber.like('%$query%'),
                    l.cellPhone.like('%$_num%'),
                  ),
                  or(
                    l.homePhone.like('%$_num%'),
                    l.officePhone.like('%$_num%'),
                  ));
            }
            return or(
                or(
                  l.firstName.like('%$query%'),
                  l.lastName.like('%$query%'),
                ),
                or(
                  l.loanMilestone.like('%$query%'),
                  l.email.like('%$query%'),
                ));
          })
          ..orderBy([
            (u) => OrderingTerm(expression: u.lastName, mode: OrderingMode.asc),
            (u) =>
                OrderingTerm(expression: u.firstName, mode: OrderingMode.asc),
          ]))
        .get();
  }

  Future<Loan> getLastLoan(String userID, api.LoansFolder folder) async {
    final _items = await (select(loanRecords)
          ..where((l) => l.userID.equals(userID))
          ..where((l) => l.folder.equals(describeEnum(folder)))
          ..orderBy([
            (u) => OrderingTerm(
                expression: u.lastUpdated, mode: OrderingMode.desc),
          ]))
        .get();
    if (_items != null && _items.isNotEmpty) {
      return _items.first;
    }
    return null;
  }

  Future<Loan> getLoan(String userID, String loanID) async {
    final _loans = await loansById(userID, loanID);
    if (_loans != null && _loans.isNotEmpty) {
      return _loans.first;
    }
    return null;
  }

  Future resetLoansForUser(String userID) {
    return (delete(loanRecords)..where((t) => t.userID.equals(userID))).go();
  }

  Future cleanLoans(
    String userID,
    DateTime lastUpdated, {
    api.LoansFolder folder = api.LoansFolder.all,
  }) {
    return (delete(loanRecords)
          ..where((t) => t.userID.equals(userID))
          ..where((t) => t.folder.equals(describeEnum(folder)))
          ..where((l) => l.lastUpdated.isBiggerThanValue(lastUpdated)))
        .go();
  }

  /// Get a Phone Number from a 10 digit string (after symbols removed)
  String phoneFormat(String value) {
    String areaCode, prefix, number, ext;
    var _number = value;
    if (_number.length >= 10) {
      areaCode = _number.substring(0, 3);
      prefix = _number.substring(3, 6);
      number = _number.substring(6, 10);
      if (_number.length > 10) {
        ext = value.substring(10, _number.length);
      }
      if (_number.isEmpty) return "";
      if (ext != null) {
        print('($areaCode) $prefix-$number :$ext');
        return '($areaCode) $prefix-$number :$ext'.toString();
      }
      print('($areaCode) $prefix-$number');
      return '($areaCode) $prefix-$number'.toString();
    }
    return '';
  }
}
