import 'package:bloc/bloc.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get_it/get_it.dart';
import 'package:intl/date_symbol_data_local.dart';

import 'data/blocs/alerts/bloc.dart';
import 'data/blocs/bloc.dart';
import 'data/blocs/videos/videos_bloc.dart';
import 'data/services/database/database.dart';
import 'data/services/database/database/shared.dart';
import 'data/services/database/tables/settings.dart';
import 'data/utils/utils.dart';
import 'plugins/desktop/desktop.dart';
import 'ui/account/screen.dart';
import 'ui/auth/screen.dart';
import 'ui/contacts/screen.dart';
import 'ui/home/screen.dart';
import 'ui/leads/screen.dart';
import 'ui/loans/screen.dart';
import 'ui/search/screen.dart';
import 'ui/settings/builder.dart';
import 'ui/video/screen.dart';

GetIt getIt = GetIt.instance;

void main() {
  BlocSupervisor.delegate = SimpleBlocDelegate();
  getIt.registerSingleton<DatabaseService>(constructDb());
  setTargetPlatformForDesktop();
  initializeDateFormatting().then((_) => runApp(MyApp()));
}

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  final _auth = AuthBloc();
  final _nav = GlobalKey<NavigatorState>();
  final _scaffoldKey = GlobalKey<ScaffoldState>();

  @override
  void initState() {
    _auth.dispatch(CheckUser());
    super.initState();
  }

  @override
  void dispose() {
    _auth.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return MultiBlocProvider(
      providers: [
        BlocProvider<NavBloc>(builder: (_) => NavBloc(_nav)),
        BlocProvider<ScaffoldBloc>(builder: (_) => ScaffoldBloc(_scaffoldKey)),
        BlocProvider<AuthBloc>(builder: (_) => _auth),
        BlocProvider<LoansBloc>(builder: (_) => LoansBloc()),
        BlocProvider<SearchBloc>(builder: (_) => SearchBloc()),
        BlocProvider<ContactsBloc>(builder: (_) => ContactsBloc()),
        BlocProvider<AlertsBloc>(builder: (_) => AlertsBloc()),
        BlocProvider<LeadsBloc>(builder: (_) => LeadsBloc()),
        BlocProvider<CalendarBloc>(builder: (_) => CalendarBloc()),
        BlocProvider<VideosBloc>(builder: (_) => VideosBloc()),
      ],
      child: SettingBuilder(
        type: SettingTypes.darkMode,
        builder: (context, setting, val, string) => MaterialApp(
          navigatorKey: _nav,
          debugShowCheckedModeBanner: false,
          title: 'My Unify CRM',
          theme: val ? AppColors.dark : AppColors.light,
          home: AuthCheck(scaffoldKey: _scaffoldKey),
          onGenerateRoute: onGenerateRoute,
        ),
      ),
    );
  }

  static Route onGenerateRoute(RouteSettings settings) {
    final path = settings.name;
    final Map<String, dynamic> args = settings?.arguments;
    switch (path) {
      case LoginScreen.routeName:
        if (args != null) {
          final bool _add = args['add'];
          return _buildRoute(settings, LoginScreen(addAccount: _add ?? false));
        }
        return _buildRoute(settings, LoginScreen());
      case SearchScreen.routeName:
        if (args != null) {
          final int _index = args['type'];
          return _buildRoute(
            settings,
            SearchScreen(
              scopes: _index == null ? null : [SearchResultType.values[_index]],
            ),
          );
        }
        return _buildRoute(settings, SearchScreen());
      case LoansScreen.routeName:
        return _buildRoute(settings, LoansScreen());
      case AccountScreen.routeName:
        return _buildRoute(settings, AccountScreen());
      case ContactsScreen.routeName:
        return _buildRoute(settings, ContactsScreen());
      case LeadsScreen.routeName:
        return _buildRoute(settings, LeadsScreen());
      case VideosScreen.routeName:
        return _buildRoute(settings, VideosScreen());
      default:
        return null;
    }
  }

  static MaterialPageRoute _buildRoute(RouteSettings settings, Widget builder) {
    return MaterialPageRoute(
      settings: settings,
      builder: (context) => builder,
    );
  }
}

class AuthCheck extends StatelessWidget {
  const AuthCheck({
    @required this.scaffoldKey,
    Key key,
  }) : super(key: key);

  final GlobalKey<ScaffoldState> scaffoldKey;

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<AuthState>(
        stream: BlocProvider.of<AuthBloc>(context).state,
        builder: (context, snapshot) {
          if (snapshot.hasData) {
            final state = snapshot.data;
            if (state is LoggedInState) {
              return HomeScreen(scaffoldKey: scaffoldKey);
            }
          }
          return LoginScreen();
        });
  }
}
