import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:unify_api/unify.dart';

import '../../data/blocs/bloc.dart';
import '../../data/utils/size.dart';
import '../../plugins/url_launcher/url_launcher.dart';
import '../../r.dart';
import 'common/index.dart';

class LoginScreen extends StatefulWidget {
  const LoginScreen({Key key, this.addAccount = false}) : super(key: key);

  static const String routeName = '/auth';

  final bool addAccount;

  @override
  _LoginScreenState createState() => _LoginScreenState();
}

class _LoginScreenState extends State<LoginScreen> {
  String _username, _password, _companyCode;
  final _formKey = GlobalKey<FormState>();
  bool _hidePassword = true;

  Widget _buildLeftSide(AuthBloc auth) {
    return SafeArea(
      bottom: false,
      child: LayoutBuilder(
        builder: (context, dimens) => StreamBuilder<AuthState>(
            stream: auth.state,
            builder: (context, snapshot) {
              final state = snapshot.data;
              if (dimens.maxHeight < 700) {
                return SingleChildScrollView(
                  child: Column(
                    children: <Widget>[
                      _buildForm(auth, context, state),
                      Container(height: 30),
                      _buildFooter(dimens, context, null),
                    ],
                  ),
                );
              }
              return Column(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  Expanded(
                    child: _buildForm(auth, context, state),
                  ),
                  _buildFooter(dimens, context, Colors.grey[200]),
                ],
              );
            }),
      ),
    );
  }

  Widget _buildFooter(
      BoxConstraints dimens, BuildContext context, Color color) {
    return Container(
      width: dimens.maxWidth,
      color: color,
      child: Container(
          padding: EdgeInsets.all(10.0),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            children: <Widget>[
              Container(
                // padding: EdgeInsets.only(left: 14.0),
                child: Text.rich(
                  TextSpan(
                    children: [
                      TextSpan(
                        text: 'Need assistance?  ',
                        style: TextStyle(fontWeight: FontWeight.bold),
                      ),
                      TextSpan(text: ' Support@CrossMedia-llc.com'),
                      TextSpan(text: ' | '),
                      TextSpan(text: ' 866-497-3224 '),
                    ],
                  ),
                ),
              ),
              Wrap(
                // mainAxisAlignment: MainAxisAlignment.spaceAround,
                children: <Widget>[
                  FlatButton(
                    child: Text('Privacy Policy'),
                    onPressed: () => showDialog(
                      context: context,
                      builder: (_) => AlertDialog(
                        title: Text('Privacy Policy'),
                        content: PrivacyPolicyView(),
                        actions: <Widget>[
                          FlatButton(
                            child: Text('OK'),
                            onPressed: () => Navigator.maybePop(context),
                          ),
                        ],
                      ),
                    ),
                  ),
                  FlatButton(
                    child: Text('Terms & Conditions'),
                    onPressed: () => showDialog(
                      context: context,
                      builder: (_) => AlertDialog(
                        title: Text('Terms & Conditions'),
                        content: TermsAndConditionsView(),
                        actions: <Widget>[
                          FlatButton(
                            child: Text('OK'),
                            onPressed: () => Navigator.maybePop(context),
                          ),
                        ],
                      ),
                    ),
                  ),
                ],
              )
            ],
          )),
    );
  }

  Widget _buildForm(AuthBloc auth, BuildContext context, AuthState state) {
    return Form(
      key: _formKey,
      child: Column(
        children: <Widget>[
          Container(
            padding: EdgeInsets.only(top: 20.0),
            height: 150,
            child: Image.asset(
              R.assetsImagesUnifyLogo,
              fit: BoxFit.contain,
            ),
          ),
          ListTile(
            title: TextFormField(
              decoration: InputDecoration(
                prefixIcon: Icon(Icons.email),
                hintText: 'Username',
              ),
              validator: (val) => val.isNotEmpty ? null : 'Username Required!',
              onSaved: (val) => _username = val,
            ),
          ),
          ListTile(
            title: TextFormField(
              decoration: InputDecoration(
                  prefixIcon: Icon(Icons.lock),
                  hintText: 'Password',
                  suffixIcon: IconButton(
                    icon: Icon(_hidePassword
                        ? Icons.visibility
                        : Icons.visibility_off),
                    onPressed: () {
                      if (mounted)
                        setState(() {
                          _hidePassword = !_hidePassword;
                        });
                    },
                  )),
              validator: (val) => val.isNotEmpty ? null : 'Password Required!',
              obscureText: _hidePassword,
              onSaved: (val) => _password = val,
            ),
          ),
          ListTile(
            title: TextFormField(
              initialValue: 'mobile.myunifycrm.com',
              decoration: InputDecoration(
                prefixIcon: Icon(Icons.info),
                hintText: 'Company Code',
              ),
              validator: (val) =>
                  val.isNotEmpty ? null : 'Company Code Required!',
              onSaved: (val) => _companyCode = val,
            ),
          ),
          ListTile(
            title: RaisedButton(
              child: Text(
                'Login',
                style: TextStyle(color: Colors.white),
              ),
              color: Theme.of(context).accentColor,
              onPressed: () async {
                if (_formKey.currentState.validate()) {
                  _formKey.currentState.save();
                  auth.dispatch(LoginEvent(
                    _username,
                    _password,
                    _companyCode,
                    addAccount: widget.addAccount,
                    navBloc: BlocProvider.of<NavBloc>(context),
                  ));
                  if (widget.addAccount) {
                    final _client = UnifyClient(_username, _password,
                        domain: _companyCode.contains('mobile.')
                            ? _companyCode
                            : 'mobile.$_companyCode');
                    await _client.login();
                    if (_client.isAuthorized) {
                      await Navigator.of(context).maybePop();
                    }
                  }
                }
              },
            ),
          ),
          Container(
            child: Flex(
              direction: Axis.horizontal,
              mainAxisAlignment: MainAxisAlignment.spaceAround,
              children: <Widget>[
                Flexible(
                  flex: 1,
                  child: FlatButton(
                    child: Text('Forgot Username?'),
                    onPressed: () {},
                  ),
                ),
                Flexible(
                  flex: 1,
                  child: FlatButton(
                    child: Text('Forgot Password?'),
                    onPressed: () {},
                  ),
                ),
              ],
            ),
          ),
          if (kIsWeb) ...[
            Container(
              padding: EdgeInsets.only(top: 10.0),
              child: Flex(
                direction: Axis.horizontal,
                mainAxisAlignment: MainAxisAlignment.spaceAround,
                children: <Widget>[
                  Flexible(
                    flex: 1,
                    child: FlatButton(
                      child: Image.asset(R.assetsImagesMobileAppstore),
                      onPressed: () => UrlUtils.open(
                          'https://apps.apple.com/us/app/my-unify-crm/id1448247041?ls=1',
                          name: 'My Unify Mobile'),
                    ),
                  ),
                  Flexible(
                    flex: 1,
                    child: FlatButton(
                      child: Image.asset(
                        R.assetsImagesMobileGoogleplay,
                        fit: BoxFit.contain,
                      ),
                      onPressed: () => UrlUtils.open(
                          'https://play.google.com/store/apps/details?id=com.crossmediallc.myunifycrm',
                          name: 'My Unify Mobile'),
                    ),
                  ),
                ],
              ),
            )
          ],
          if (state is LogInErrorState)
            ListTile(
              title: Text(state.error),
            ),
          if (state is LogInLoadingState)
            Center(child: CircularProgressIndicator()),
        ],
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final auth = BlocProvider.of<AuthBloc>(context);
    return Scaffold(
      appBar: widget.addAccount ? AppBar() : null,
      body: LayoutBuilder(
        builder: (context, dimens) {
          final _isDesktop = dimens.maxWidth > kBreakpoint;
          return Flex(
            direction: Axis.horizontal,
            children: <Widget>[
              Flexible(
                flex: 1,
                child: Container(
                  child: _buildLeftSide(auth),
                ),
              ),
              if (_isDesktop)
                Flexible(
                    flex: 2,
                    child: Stack(
                      fit: StackFit.expand,
                      children: <Widget>[
                        Positioned.fill(
                          child: Image.asset(
                            R.assetsImagesLoginBg,
                            fit: BoxFit.cover,
                          ),
                        ),
                        Positioned(
                          top: 200,
                          left: 20.0,
                          child: Text(
                            'What will you\naccomplish today\nwith Unify?',
                            style: TextStyle(
                              fontSize: 50,
                              color: Colors.white,
                              fontWeight: FontWeight.w500,
                            ),
                          ),
                        ),
                      ],
                    )),
            ],
          );
        },
      ),
    );
  }
}
