import 'package:after_layout/after_layout.dart';
import 'package:flutter/material.dart';
import 'package:table_calendar/table_calendar.dart';

import '../../plugins/calendar/calendar.dart';
import '../../plugins/calendar/classes/calender.dart';
import 'common/index.dart';

class CalendarScreen extends StatefulWidget {
  @override
  _CalendarScreenState createState() => _CalendarScreenState();
}

class _CalendarScreenState extends State<CalendarScreen>
    with TickerProviderStateMixin, AfterLayoutMixin<CalendarScreen> {
  AnimationController _animationController;
  final CalendarController _calendarController = CalendarController();

  DateTime _end = DateTime.now().add(Duration(days: 30));
  Map<DateTime, List> _events = <DateTime, List>{};
  List _selectedEvents = [];
  DateTime _start = DateTime.now().subtract(Duration(days: 30));

  @override
  void afterFirstLayout(BuildContext context) {
    _calendarController.setSelectedDay(DateTime.now());
  }

  @override
  void dispose() {
    _animationController.dispose();
    _calendarController.dispose();
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    CalendarUtils.requestPermissions();
    _loadCalendars();
    _animationController = AnimationController(
      vsync: this,
      duration: const Duration(milliseconds: 400),
    );
    _refreshEvents(DateTime.now());
  }

  void _refreshEvents(DateTime date, {bool force = false}) async {
    final _avaliable = await CalendarUtils.hasPermissions();
    if (_avaliable != null && _avaliable) {
      final _data = await CalendarUtils.getCalendars();
      if (_data != null) {
        for (var cal in _data) {
          const days = 60;
          await _retrieveCalendarEvents(
            cal.id,
            days,
            start: date.subtract(Duration(days: (days / 2).round())),
            end: date.add(Duration(days: (days / 2).round())),
            force: force,
          );
        }
      } else {
        print('No Calendars Found');
      }
    } else {
      print('No Calendar Permissions');
    }
  }

  Future _retrieveCalendarEvents(
    String _calendar,
    int days, {
    @required DateTime start,
    @required DateTime end,
    bool force = false,
  }) async {
    final calendarEventsResult = await CalendarUtils.getEventsForCalendar(
        _calendar,
        startDate: start,
        endDate: end);
    final _calendarEvents = calendarEventsResult;
    if (force) {
      _events = {};
    }

    for (var event in _calendarEvents) {
      if (_events[event.start] == null) {
        _events[event.start] = [];
      }
      _events[event.start].add(event.title);
    }
    if (mounted)
      setState(() {
        _animationController.forward();
      });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      // appBar: AppBar(
      //   leading: MenuButton(),
      //   title: Text('Calendar'),
      // ),
      body: LayoutBuilder(
        builder: (context, dimens) {
          if (dimens.maxWidth > 600) {
            return Row(
              children: <Widget>[
                Container(
                  width: 400,
                  child: _buildCal(),
                ),
                Expanded(
                  child: Container(child: _buildEvents()),
                ),
              ],
            );
          }
          return Column(
            mainAxisSize: MainAxisSize.max,
            children: <Widget>[
              _buildCal(),
              Expanded(child: _buildEvents()),
            ],
          );
        },
      ),
    );
  }

  Widget _buildEvents() {
    return Column(
      children: <Widget>[
        const SizedBox(height: 8.0),
        if (_selectedEvents != null)
          Expanded(
              child: ListView(
            children: _selectedEvents
                .map((event) => Container(
                      decoration: BoxDecoration(
                        border: Border.all(width: 0.8),
                        borderRadius: BorderRadius.circular(12.0),
                      ),
                      margin: const EdgeInsets.symmetric(
                          horizontal: 8.0, vertical: 4.0),
                      child: ListTile(
                        title: Text(event.toString()),
                        onTap: () => print('$event tapped!'),
                      ),
                    ))
                .toList(),
          )),
      ],
    );
  }

  List<DeviceCal> _calendars = [];

  void _loadCalendars() async {
    final _data = await CalendarUtils.getCalendars();
    if (mounted)
      setState(() {
        _calendars = _data;
      });
  }

  Widget _buildCal() {
    return Column(
      children: <Widget>[
        UnifyCalendar(
          calendars: _calendars,
          calendarController: _calendarController,
          start: _start,
          end: _end,
          onDaySelected: (day, events) {
            if (mounted) {
              setState(() {
                _selectedEvents = events;
                _calendarController.setSelectedDay(day);
              });
            }
          },
          onVisibleDaysChanged: (first, last, format) {
            print('CALLBACK: _onVisibleDaysChanged => $format');
          },
        ),
        const SizedBox(height: 8.0),
        Row(
          mainAxisAlignment: MainAxisAlignment.spaceAround,
          children: <Widget>[
            ActionChip(
              avatar: Icon(Icons.share),
              label: Text('Share'),
              onPressed: () {},
            ),
            ActionChip(
              avatar: Icon(Icons.today),
              label: Text('Today'),
              onPressed: () {
                _calendarController.setSelectedDay(DateTime.now(),
                    runCallback: true);
              },
            ),
            ActionChip(
              avatar: Icon(Icons.refresh),
              label: Text('Refresh'),
              onPressed: () =>
                  _refreshEvents(_calendarController.selectedDay, force: true),
            ),
          ],
        ),
      ],
    );
  }
}
