import 'package:auto_size_text/auto_size_text.dart';
import 'package:flutter/material.dart';
import 'package:unify_api/unify.dart' as api;

import '../../../data/blocs/bloc.dart';
import '../../../data/services/database/database.dart';
import '../../../data/utils/utils.dart';
import '../../../plugins/local_storage/local_storage.dart';
import '../../common/raised_card.dart';

class DashboardTotals extends StatefulWidget {
  static const String cardKey = 'card_totals';
  final String tag;

  const DashboardTotals({Key key, this.tag}) : super(key: key);

  @override
  _DashboardTotalsState createState() => _DashboardTotalsState();
}

class _DashboardTotalsState extends State<DashboardTotals> {
  LocalStor storage;

  @override
  void initState() {
    storage = LocalStor(widget?.tag ?? 'totals');
    super.initState();
  }

  String _totals;
  num _rank, _apps, _totalV, _units, _efficency;

  void init() async {
    _totals = await storage.getSavedItem('totals');
    _rank = await storage.getSavedItem('rank');
    _apps = await storage.getSavedItem('applications');
    _totalV = await storage.getSavedItem('totalV');
    _units = await storage.getSavedItem('units');
    _efficency = await storage.getSavedItem('efficiency');
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    final user = AuthBloc.currentUser(context);
    final _client =
        api.UnifyClient(user.username, user.password, domain: user.domain);

    if (_totals == null) {
      getStats(_client, user);
    }
    return Padding(
      padding: const EdgeInsets.only(left: 10, right: 10),
      child: RaisedChild(LayoutBuilder(
        builder: (context, constraints) => Container(
          height: 200,
          width: constraints.maxWidth > kBreakpoint
              ? constraints.maxWidth * 0.2
              : constraints.maxWidth,
          padding: const EdgeInsets.all(24.0),
          child: FutureBuilder(
              future: storage.init(),
              builder: (context, loaded) {
                return Stack(
                  children: <Widget>[
                    Center(
                      child: CircleAvatar(
                        backgroundColor: Colors.red,
                        maxRadius: 50,
                        child: InkWell(
                          onTap: () => getStats(_client, user),
                          child: _RankWidget(
                            name: "Your Rank",
                            textAlign: TextAlign.center,
                            value: "# ${_rank.toStringAsFixed(0) ?? 0}",
                            nameColor: Colors.white,
                            up: true,
                          ),
                        ),
                      ),
                    ),
                    Positioned(
                        left: 0,
                        bottom: 0,
                        top: 0,
                        child: Column(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: <Widget>[
                            Container(
                              width: 100,
                              child: _RankWidget(
                                name: "Applications",
                                textAlign: TextAlign.left,
                                value: "${_apps.toStringAsFixed(0) ?? 0}",
                                up: true,
                              ),
                            ),
                            Container(
                              width: 100,
                              child: _RankWidget(
                                name: "Total Volume",
                                textAlign: TextAlign.left,
                                value: "\$${_totalV.toStringAsFixed(2) ?? 0}",
                                up: true,
                              ),
                            )
                          ],
                        )),
                    Positioned(
                        right: 0,
                        top: 0,
                        bottom: 0,
                        child: Column(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: <Widget>[
                            Container(
                              width: 100,
                              child: _RankWidget(
                                name: "Total Units",
                                textAlign: TextAlign.right,
                                value: "${_units.toStringAsFixed(0) ?? 0}",
                                up: true,
                              ),
                            ),
                            Container(
                              width: 100,
                              child: _RankWidget(
                                name: "Efficiency",
                                value:
                                    "${_efficency.toStringAsFixed(0) ?? 0} days",
                                textAlign: TextAlign.right,
                                up: true,
                              ),
                            ),
                          ],
                        )),
                  ],
                );
              }),
        ),
      )),
    );
  }

  Future getStats(api.UnifyClient client, User user) async {
    final response = await client.getStats();
    await storage.setSavedItem('applications', response.application.value);
    await storage.setSavedItem('totalV', response.volume.value);
    await storage.setSavedItem('units', response.unitsClosed.value);
    await storage.setSavedItem('efficiency', response.efficiency.value);
    await storage.setSavedItem('rank', response.rank.value);
  }
}

class _RankWidget extends StatelessWidget {
  const _RankWidget({
    Key key,
    this.name = "",
    this.up = true,
    this.value = "0",
    this.nameColor,
    this.textAlign = TextAlign.center,
  }) : super(key: key);

  final String name;
  final String value;
  final bool up;
  final TextAlign textAlign;
  final Color nameColor;

  @override
  Widget build(BuildContext context) {
    final _textStyle = Theme.of(context)
        .textTheme
        .display1
        .copyWith(color: nameColor, fontWeight: FontWeight.w700);
    final _textColor = Colors.black;
    if (textAlign == TextAlign.right) {
      return Container(
        width: 100,
        child: Column(
          mainAxisAlignment: MainAxisAlignment.end,
          crossAxisAlignment: CrossAxisAlignment.end,
          children: <Widget>[
            AutoSizeText(
              name,
              textAlign: textAlign,
              maxLines: 1,
              textScaleFactor: 1,
              style: TextStyle(color: Colors.red),
            ),
            AutoSizeText(
              value,
              textScaleFactor: 1,
              textAlign: textAlign,
              style: _textStyle.copyWith(color: _textColor),
              maxLines: 1,
            )
          ],
        ),
      );
    }
    if (textAlign == TextAlign.left) {
      return Container(
        width: 100,
        child: Column(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            AutoSizeText(
              name,
              textAlign: textAlign,
              maxLines: 1,
              textScaleFactor: 1,
              style: TextStyle(color: Colors.red),
            ),
            AutoSizeText(
              value,
              maxLines: 1,
              textScaleFactor: 1,
              textAlign: textAlign,
              style: _textStyle.copyWith(color: _textColor),
            )
          ],
        ),
      );
    }
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        AutoSizeText(
          name,
          textAlign: textAlign,
          maxLines: 1,
          textScaleFactor: 1,
          style: TextStyle(color: Colors.white),
        ),
        AutoSizeText(
          value,
          textScaleFactor: 1,
          textAlign: textAlign,
          maxLines: 1,
          style: _textStyle,
        ),
      ],
    );
  }
}
