import 'dart:convert';

import 'package:example_db/ui/search/screen.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get_it/get_it.dart';
import 'package:intl/intl.dart';
import 'package:json_table/json_table.dart';
import 'package:json_table/json_table_column.dart';
import 'package:moor/moor.dart' hide Column;
import 'package:recase/recase.dart';

import '../../../data/blocs/bloc.dart';
import '../../../data/classes/enum_list.dart';
import '../../../data/services/database/database.dart';
import 'tile.dart';

class LeadsListScreen extends StatefulWidget {
  @override
  _LeadsListScreenState createState() => _LeadsListScreenState();

  static refresh(BuildContext context) {
    final leads = BlocProvider.of<LeadsBloc>(context);
    final _user = AuthBloc.currentInfo(context);
    final scaffold = BlocProvider.of<ScaffoldBloc>(context);
    return leads.dispatch(
        RefreshLeads(user: _user, scaffold: scaffold, context: context));
  }
}

class _LeadsListScreenState extends State<LeadsListScreen> {
  int _sortColumnIndex = 1;
  bool _sortAccending = true;
  @override
  Widget build(BuildContext context) {
    final leads = BlocProvider.of<LeadsBloc>(context);
    final _user = AuthBloc.currentUser(context);

    return Scaffold(
      // appBar: AppBar(
      //   title: Text('Leads'),
      //   actions: <Widget>[
      //     IconButton(
      //         icon: Icon(Icons.refresh), onPressed: () => refresh(context)),
      //   ],
      // ),
      floatingActionButton: FloatingActionButton(
        heroTag: 'search_leads',
        child: Icon(Icons.search),
        onPressed: () {
          Navigator.of(context).pushNamed(SearchScreen.routeName,
              arguments: <String, dynamic>{
                'type': SearchResultType.leads.index
              });
        },
      ),
      floatingActionButtonLocation: FloatingActionButtonLocation.endDocked,
      bottomNavigationBar: BottomAppBar(
        child: Container(
          child: Row(
            children: <Widget>[],
          ),
        ),
      ),
      body: StreamBuilder<LeadsState>(
        stream: leads.state,
        builder: (context, snapshot) {
          final _state = snapshot.data;
          return Column(
            children: <Widget>[
              if (_state is LeadsLoadingState) LinearProgressIndicator(),
              Expanded(
                  child: Container(
                child: StreamBuilder<List<Lead>>(
                    stream: GetIt.instance<DatabaseService>()
                        .leadsDao
                        .watchLeadsByUser(_user.userID),
                    builder: (context, snapshot) {
                      if (snapshot.hasData) {
                        final _leads = snapshot.data;
                        if (_leads.isEmpty) {
                          if (_state is LeadsLoadingState) {
                            return Column(
                              mainAxisAlignment: MainAxisAlignment.center,
                              crossAxisAlignment: CrossAxisAlignment.center,
                              children: [
                                Text('No Leads Found'),
                                SizedBox(height: 20.0),
                                CircularProgressIndicator(),
                              ],
                            );
                          }
                          return Center(child: Text('No Leads Found'));
                        }

                        return Scrollbar(
                          child: RefreshIndicator(
                            child: ListView.separated(
                              separatorBuilder: (context, index) =>
                                  Divider(height: 1),
                              itemCount: _leads.length,
                              itemBuilder: (context, index) {
                                final _lead = _leads[index];
                                return Padding(
                                  padding: EdgeInsets.all(7),
                                  child: LeadRowTile(
                                    lead: _lead,
                                    type: ViewType.detailed,
                                  ),
                                );
                              },
                            ),
                            onRefresh: () async {
                              LeadsListScreen.refresh(context);
                              await Future.delayed(Duration(seconds: 3));
                            },
                          ),
                        );
                      }
                      return Center(child: CircularProgressIndicator());
                    }),
              ))
            ],
          );
        },
      ),
    );
  }

  static String _getDate(val) {
    try {
      const serializer = const ValueSerializer.defaults();
      final _date = serializer.fromJson<DateTime>(val);
      final _format = DateFormat.yMd();
      return _format.format(_date);
    } catch (e) {}
    return val.toString();
  }

  Widget _buildHeader(
      String header, BuildContext context, List<JsonTableColumn> columns) {
    final _index = columns.indexWhere((c) => c.label == header);
    return GestureDetector(
      onTap: () {
        if (mounted)
          setState(() {
            if (_sortColumnIndex == _index) {
              _sortAccending = !_sortAccending;
            } else {
              _sortColumnIndex = _index;
            }
          });
      },
      child: Row(
        children: <Widget>[
          Expanded(
            child: Container(
              padding: EdgeInsets.symmetric(horizontal: 8.0, vertical: 4.0),
              decoration: BoxDecoration(
                  border: Border.all(width: 0.5), color: Colors.grey[300]),
              child: Text(
                header,
                textAlign: TextAlign.center,
                style: Theme.of(context).textTheme.display1.copyWith(
                    fontWeight: FontWeight.w700,
                    fontSize: 14.0,
                    color: Colors.black87),
              ),
            ),
          ),
          if (_sortColumnIndex == _index)
            Container(
              child: Column(
                children: <Widget>[
                  Icon(
                    _sortAccending ? Icons.arrow_upward : Icons.arrow_downward,
                    color: (Theme.of(context).brightness == Brightness.light)
                        ? Colors.black87
                        : Colors.white70,
                  ),
                ],
              ),
            ),
        ],
      ),
    );
  }

  Widget _buildCell(value, BuildContext context) {
    return Container(
      padding: EdgeInsets.symmetric(horizontal: 4.0, vertical: 2.0),
      decoration: BoxDecoration(
          border: Border.all(width: 0.5, color: Colors.grey.withOpacity(0.5))),
      child: Text(
        value,
        textAlign: TextAlign.center,
        style: Theme.of(context)
            .textTheme
            .display1
            .copyWith(fontSize: 14.0, color: Colors.grey[900]),
      ),
    );
  }
}

class ColumnSelect extends StatefulWidget {
  const ColumnSelect({
    Key key,
    @required this.allColumns,
    @required this.selectedColumns,
    @required this.columnsChanged,
  }) : super(key: key);

  final List<String> allColumns;
  final List<String> selectedColumns;
  final ValueChanged<List<String>> columnsChanged;

  @override
  _ColumnSelectState createState() => _ColumnSelectState();
}

class _ColumnSelectState extends State<ColumnSelect> {
  List<String> selectedColumns;

  @override
  void initState() {
    selectedColumns = widget.selectedColumns;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        for (var item in widget.allColumns)
          CheckboxListTile(
            title: Text(ReCase(item).titleCase),
            value: selectedColumns.contains(item),
            onChanged: (val) {
              if (mounted)
                setState(() {
                  if (val) {
                    selectedColumns.add(item);
                  } else {
                    selectedColumns.remove(item);
                  }
                });
              widget.columnsChanged(selectedColumns);
            },
          ),
      ],
    );
  }
}
